@@ -6,4 +6,4 @@ import sys
 if sys.version_info[0] == 3:
     from multiprocessing.connection import *  # noqa
 else:
-    from ._connection import *                # noqa
+    from billiard._connection import *        # noqa
