@@ -29,7 +29,8 @@ import warnings
 
 from . import Event, Process, cpu_count
 from . import util
-from .common import restart_state
+from .common import reset_signals, restart_state
+from .compat import get_errno
 from .einfo import ExceptionInfo
 from .exceptions import (
     CoroStop,
@@ -62,17 +63,18 @@ except NameError:
                 raise
             return args[0]
 
+PY3 = sys.version_info[0] == 3
 
 try:
     TIMEOUT_MAX = threading.TIMEOUT_MAX
-except AttributeError:
-    TIMEOUT_MAX = 1e10
+except AttributeError:  # pragma: no cover
+    TIMEOUT_MAX = 1e10  # noqa
 
 
-try:
-    _Semaphore = threading._Semaphore
-except AttributeError:  # Py3
-    _Semaphore = threading.Semaphore  # noqa
+if PY3:
+    _Semaphore = threading.Semaphore
+else:
+    _Semaphore = threading._Semaphore  # noqa
 
 #
 # Constants representing the state of a pool
@@ -133,11 +135,17 @@ class LaxBoundedSemaphore(_Semaphore):
     but ignores if # releases >= value."""
 
     def __init__(self, value=1, verbose=None):
-        _Semaphore.__init__(self, value, verbose)
+        if PY3:
+            _Semaphore.__init__(self, value)
+        else:
+            _Semaphore.__init__(self, value, verbose)
         self._initial_value = value
 
     def grow(self):
-        cond = self._Semaphore__cond
+        if PY3:
+            cond = self._cond
+        else:
+            cond = self._Semaphore__cond
         with cond:
             self._initial_value += 1
             self._Semaphore__value += 1
@@ -147,10 +155,10 @@ class LaxBoundedSemaphore(_Semaphore):
         self._initial_value -= 1
         self.acquire()
 
-    if sys.version_info[0] == 3:
+    if PY3:
 
         def release(self):
-            cond = self._Semaphore__cond
+            cond = self._cond
             with cond:
                 if self._value < self._initial_value:
                     self._value += 1
@@ -262,6 +270,10 @@ def worker(inqueue, outqueue, initializer=None, initargs=(),
     if initializer is not None:
         initializer(*initargs)
 
+    # Make sure all exiting signals call finally: blocks.
+    # this is important for the semaphore to be released.
+    reset_signals()
+    # install signal handler for soft timeouts.
     if SIG_SOFT_TIMEOUT is not None:
         signal.signal(SIG_SOFT_TIMEOUT, soft_timeout_sighandler)
 
@@ -277,7 +289,9 @@ def worker(inqueue, outqueue, initializer=None, initargs=(),
             ready, task = poll(1.0)
             if not ready:
                 continue
-        except (EOFError, IOError):
+        except (EOFError, IOError), exc:
+            if get_errno(exc) == errno.EINTR:
+                continue  # interrupted, maybe by gdb
             debug('worker got EOFError or IOError -- exiting')
             exitcode = EX_FAILURE
             break
@@ -484,7 +498,7 @@ class TimeoutHandler(PoolThread):
         try:
             _kill(job._worker_pid, SIG_SOFT_TIMEOUT)
         except OSError, exc:
-            if exc.errno != errno.ESRCH:
+            if get_errno(exc) != errno.ESRCH:
                 raise
 
     def on_hard_timeout(self, job):
